/*   options.c --- Ants Colony
 *   
 *   Copyright (C) 2000 Valvassori Mose <djedi@altern.org>
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA **/

#include <dtk/dtk.h>
#include <math.h>
#include <stdio.h>
#include "ants.h"

Form *optionw;
static Form *oantsf;
static Form *oantnbt;

static Form *owindf,*owindsw;
static Form *owinda_tt,*owinds_tt;

#ifdef HAVE_LIBJPEG
static Form *oimgf;
#endif

static Form *oworldf;
static Form *oworldnbf,*owf1;

static int real_time=1;

static char tmpstring[20];					 /* Beware */



static char *
inttochar(int i)
{
  sprintf(tmpstring,"%d",i);
  return (tmpstring);
}

static char *
floattochar(float i)
{
  sprintf(tmpstring,"%.2f",i);
  return (tmpstring);
}

/********************* 
 *  Callbacks	     * 
 *********************/
static void
windswr(Form *i)
{
  default_redraw(i);
  color(i,0);
  circle(i,em(10),em(10),em(20)-5);
  line(i,em(10),em(10),em(10)+em(8)*cos(wind_angle),em(10)-em(8)*sin(wind_angle));
}



static void
windcb(Form *i)
{
    top_window(owindf);
}

static void
antscb(Form *i)
{
    top_window(oantsf);
}


#ifdef HAVE_LIBJPEG
static void
imgcb(Form *i)
{
    top_window(oimgf);
}
#endif

static void
anglecb(Form *i)
{
  int p;
  p=get_slider_position(i);
  wind_angle=2*p*M_PI/SLIDER_MAX;
  wind(wind_angle,wind_strength);
  windswr(owindsw);
  set_text(owinda_tt,floattochar(wind_angle));
}

static void
strengthcb(Form *i)
{
  int p;
  p=get_slider_position(i);
  wind_strength=(float)p/SLIDER_MAX;
  wind(wind_angle,wind_strength);  
  set_text(owinds_tt,floattochar(wind_strength));
}

static void
nbantscb(Form *i)
{
  nbants = get_slider_position(i)*MAX_ANTS/SLIDER_MAX;
  set_text(oantnbt,inttochar(nbants));
}
static void
nbfoodcb(Form *i)
{
  nbfood = get_slider_position(i)*MAX_FOOD/SLIDER_MAX;
  set_text(oworldnbf,inttochar(nbfood));
}

static void
optioncancel(Form *i)
{
  close_window(optionw);
}


static void
optionrandom(Form *i)
{
  use_pheronom = (get_check(i)==true?0:1);
}
static void
optionnoise(Form *i)
{
  use_noise = (get_check(i)==true?1:0);
}
static void
optiondrawphero(Form *i)
{
  draw_pheromon = (get_check(i)==true?1:0);
}


#ifdef HAVE_LIBJPEG
static void
optionsavejpeg(Form *i)
{
  save_jpeg = (get_check(i)==true?1:0);
}
#endif

static void
worldcb(Form *i)
{
    top_window(oworldf);
}
static void
fdr0cb(Form *i)
{
    food_repartion =0;
}
static void
fdr1cb(Form *i)
{
    food_repartion =1;
}
static void
fdr2cb(Form *i)
{
 food_repartion =2;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Iniialize the options Window
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
init_options_window(void)
{
  optionw=open_window(0,0,em(60),em(50),"Ants Colony - Options"
		      ,default_redraw);

  button(optionw,em(49),em(46),em(10),em(3),"Cancel",optioncancel);
  disable(button(optionw,em(38),em(46),em(10),em(3),"Apply",nop));
  disable(button(optionw,em(27),em(46),em(10),em(3),"Done",nop));
  disable(check(optionw,em(1),em(46),em(20),em(3),"Real time",true));

  button(optionw,em(2),em(1),em(5),em(3),"Ants",antscb);
  button(optionw,em(8),em(1),em(6),em(3),"Wind",windcb);
  button(optionw,em(15),em(1),em(7),em(3),"World",worldcb);
#ifdef HAVE_LIBJPEG
  button(optionw,em(23),em(1),em(7),em(3),"Img",imgcb);
#endif

  /* Wind sub window */
  owindf = box(optionw,em(1),em(4),em(58),em(41),normal); 
  text (owindf,em(25),em(1),em(8),em(2),"Wind",center);
  text (owindf,em(2),em(6.5),em(7),em(1.5),"Angle",left);
  owinda_tt = text (owindf,em(22),em(8),em(6),em(2),floattochar(wind_angle),left);
  slider(owindf,em(1),em(8),em(20),em(2),horizontal,0,em(10),anglecb);
  text (owindf,em(2),em(14.5),em(10),em(1.5),"Strength",left);
  owinds_tt = text (owindf,em(22),em(16),em(6),em(2),floattochar(wind_strength),left);
  slider(owindf,em(1),em(16),em(20),em(2),horizontal,0,em(10),strengthcb);
  owindsw=subwindow(owindf,em(27),em(5),em(20),em(20),windswr);

#ifdef HAVE_LIBJPEG
  /* Img */
  oimgf = box(optionw,em(1),em(4),em(58),em(41),normal);
  text (oimgf,em(25),em(1),em(8),em(2),"Img",center);
  set_action(check(oimgf,em(1),em(5),em(20),em(2),"Save JPEGs",save_jpeg),optionsavejpeg);
#endif

  /* world  */
  oworldf = box(optionw,em(1),em(4),em(58),em(41),normal); 
  text (oworldf,em(25),em(1),em(8),em(2),"World",center);
  text (oworldf,em(2),em(6.5),em(15),em(1.5),"Food quantity",left);
  oworldnbf = text (oworldf,em(22),em(8),em(6),em(2),inttochar(nbfood),left);
  slider(oworldf,em(1),em(8),em(20),em(2),horizontal,(nbfood*SLIDER_MAX)/MAX_FOOD,em(10),nbfoodcb);
  owf1 = box (oworldf,em(1),em(11),em(30),em(8),normal);
  set_action(radio(owf1,em(1),em(1),em(28),em(2),"Circle",food_repartion==0?true:false),fdr0cb);
  set_action(radio(owf1,em(1),em(3),em(28),em(2),"Spiral",food_repartion==1?true:false),fdr1cb);
  set_action(radio(owf1,em(1),em(5),em(28),em(2),"Random",food_repartion==2?true:false),fdr2cb);
  set_action(check(oworldf,em(1),em(21),em(20),em(2),"Draw Pheromon",draw_pheromon),optiondrawphero);

  /* ants  */
  oantsf = box(optionw,em(1),em(4),em(58),em(41),normal); 
  text (oantsf,em(25),em(1),em(8),em(2),"Ants",center);
  text (oantsf,em(2),em(6.5),em(7),em(1.5),"Number",left);
  oantnbt = text (oantsf,em(22),em(8),em(6),em(2),inttochar(nbants),left);
  slider(oantsf,em(1),em(8),em(20),em(2),horizontal,(nbants*SLIDER_MAX)/MAX_ANTS,em(10),nbantscb);
  set_action(check(oantsf,em(1),em(12),em(20),em(2),"Random Walk",!use_pheronom),optionrandom);
  set_action(check(oantsf,em(1),em(15),em(20),em(2),"Noise",use_noise),optionnoise);
}



void
view_options_window(void)
{
  view_window(optionw);
}

