/*   ants.c --- Ants Colony
 *   
 *   Copyright (C) 2000 Valvassori Mose <djedi@altern.org>
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA **/

#define WORLD_C

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "ants.h"

unsigned char buffer1[MH][MW];
unsigned char buffer2[MH][MW];

unsigned char * pheromon1;
unsigned char * pheromon2;

int nest_pheromon[MH][MW];					 /*+ This pheromon is the distance beetwen the nest and the point +*/

int nbfood = 1;
FOOD food[MAX_FOOD];

char food_repartion = 0;
char draw_pheromon = 0;

#ifndef acces1
#define acces1(x,y) *(pheromon1+(y)*MH+(x))
#endif
#define acces2(x,y) *(pheromon2+(y)*MH+(x))


float wind_angle=.0;
float wind_strength=.0;
float c11, c21, c31, c12, c22, c32, c13, c23, c33;
float divi;

static float resonance (float a1,float a2);





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Initialise the World.
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
init_world(void)
{
  int i,j,r;
  pheromon1=(unsigned char *)buffer1;
  pheromon2=(unsigned char *)buffer2;

#ifdef HAVE_LIBJPEG
  if (image_buffer == NULL)
    image_buffer = (JSAMPLE *)calloc(MH*MW,3);
  image_height=MH;
  image_width=MW;
#endif


  for (i=0; i<nbfood; i++)
    {
      if (food_repartion==2)
	{
	  food[i].x = 10+(int)((float)(MW-20)*rand()/(RAND_MAX+1.0));
	  food[i].y = 10+(int)((float)(MH-20)*rand()/(RAND_MAX+1.0));
	}
      else
	{
	  if (food_repartion == 0)
	    r =100;
	  else
	    r = 40 + (i*100/nbfood);

	  food[i].x = NESTX + r*cos(0.2 + i * 2 * M_PI / nbfood);
	  food[i].y = NESTY + r*sin(0.2 + i * 2 * M_PI / nbfood);
	}
      food[i].rest = DEFAULT_REST;
    }

  for (j=0; j<MH; j++)
    for (i=0; i<MW; i++)
      {
	nest_pheromon[j][i]= (int)sqrt((i-NESTX)*(i-NESTX) + (j-NESTY)*(j-NESTY));
	acces1(i,j) = 0;
	acces2(i,j) = 0;
      }
}




/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Convert the world data to an X Image.
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
world2image(void)
{
  unsigned char * ppherom;
  PIXEL * pim,*p;
  int i,j,k;

#ifdef HAVE_LIBJPEG
  JSAMPLE *jpg=image_buffer+2;
#endif

  ppherom = pheromon1;
  pim = (PIXEL *)im->data;


 /*  Draw the pheromon */
  for (j=0; j<MH; j++)
    for (i=0; i<MW; i++)
      {
	if (draw_pheromon)
	  {
	    pim->msb = (*ppherom++) >> 3;
#ifdef HAVE_LIBJPEG
	    *jpg++=*ppherom;
#endif
	  }
	else
	  {
	    pim->msb = 0;
#ifdef HAVE_LIBJPEG
	    *jpg++=0;
#endif
	  }
	 /*  if ( *ppherom < 30 )
	  *  pim->lsb = 30;
	  *  else */
	pim->lsb = 0;
#ifdef HAVE_LIBJPEG
	*jpg ++=0;
	*jpg ++=0;
#endif

	pim++;
      }

  /*  Draw the foods */
  pim = (PIXEL *)im->data;
  for (i=0; i<nbfood; i++)
    {
      draw_food(pim,food[i]);
      nb_food += food[i].rest;
    }

  /*  Draw the ants */
  for (i=0; i<nbants; i++)
    {
      p = (pim+(MW*ant[i].posy)+ant[i].posx);

      k=p->msb +40;
		
		if (k>255)
		  p->msb =255;
		else
		  p->msb=(unsigned char)k;
      p->lsb = 255;

#ifdef HAVE_LIBJPEG
      jpg=1+image_buffer+3*((MW*ant[i].posy)+ant[i].posx);
      *jpg = 128+(k>>2);
      *(jpg-1) = 128+(k>>2);
#endif

      
    }
  
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  The evaparation and diffusion routine for the pheromon.
  Basicaly, I use a convolution.
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
diffusion_evaporation_pheromon()
{
  int i,j;  
  unsigned char * tmp;

  for (j=1; j<MH-1; j++)
    for (i=0; i<MW; i++)
      {
	acces2(i,j) = (c11*acces1(i-1, j-1) + c21*acces1(i, j-1) + c31*acces1(i+1,j-1) +
	  c12*acces1(i-1,j) + c22*acces1(i,j) + c32*acces1(i+1,j) +
	  c13*acces1(i-1, j+1) + c23*acces1(i, j+1) + c33*acces1(i+1,j+1)) / divi;
      }

  tmp=pheromon2;
  pheromon2=pheromon1;
  pheromon1=tmp;

  /*  memcpy(pheromon1,pheromon2,MW*MH); */
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Compute the convolution coeficient in function of wind.

  float dir the wind's direction

  float strenght the wind's strenght
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
wind (float dir, float strength)
{
  
  c11 = 1 - strength * resonance(dir,3*M_PI_4);
  c21 = 1 - strength * resonance(dir,M_PI_2);
  c31 = 1 - strength * resonance(dir,M_PI_4);

  c12 = 1 - strength * resonance(dir,-M_PI);
  c22 = 1;
  c32 = 1 - strength * resonance(dir,0);

  c13 = 1 - strength * resonance(dir,-3*M_PI_4);
  c23 = 1 - strength * resonance(dir,-M_PI_2);
  c33 = 1 - strength * resonance(dir,-M_PI_4);

  
  divi = c11+c12+c13+c21+c22+c23+c31+c32+c33;
  /*  printf(" %1.3f %1.3f %1.3f\n",c11,c21,c31);
   *  printf(" %1.3f %1.3f %1.3f\n",c12,c22,c32);
   *  printf(" %1.3f %1.3f %1.3f\n",c13,c23,c33);
   *  printf(" %1.3f\n",divi); */
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Do a resonance between two angle. Coef is greater when angles are near.

float resonnance the resonnance coeficient.

float a1 first angle

float a2 second angle
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static float
resonance (float a1,float a2)
{
  return (cos(a1)*cos(a2) + sin(a1)*sin(a2));
}

